/* Copyright 1999 by Herve Regad-Pellagru, E-mail: herve.regadpellagru@free.fr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <netdb.h>

#include <sys/socket.h>
#include <netinet/in.h>

#include "common.h"


static void usage(void)
{
  fprintf(stderr,"Usage: x10_client host port\n");
  exit(1);
}


static void fatal_error(char *msg)
{
  fprintf(stderr,msg);
  exit(1);
}


/* We connect to the server host at the specified port. return 0 on failure and 
   the connection socket on success */
static int init_client(char host[],int port)
{
  unsigned long addr;
  struct sockaddr_in them;
  struct hostent *host_ip;
  char *ip;
  int s,valsock=0;

  memset((char *)&them,0,sizeof(them));
  them.sin_family=AF_INET;
  them.sin_port=htons((unsigned short)port);

  /* We translate the host name in ip address and then in network address
     for the sockaddr_in struct */
  if(!(host_ip=gethostbyname(host)))
    fatal_error("Server name unknown !!\n");

  ip=host_ip->h_addr_list[0];

  addr=(unsigned long) ((unsigned long)ip[0]<<24L) | ((unsigned long)ip[1]<<16L) |
    ((unsigned long)ip[2]<< 8L) | ((unsigned long)ip[3]);
  them.sin_addr.s_addr=htonl(addr);

  if((s=socket(AF_INET,SOCK_STREAM,IPPROTO_TCP)) == -1)
    fatal_error("socket returned is invalid !!\n");

  if(setsockopt(s,SOL_SOCKET,SO_KEEPALIVE,(char *)&valsock,sizeof(valsock)))
    fatal_error("setsockopt keepalive failed !!\n");

  return(connect(s,(struct sockaddr *)&them,sizeof(them)) == -1 ? 0 : s);
}


/* Just a chat function which receives everything from the server, until 
   an OK. We fflush to avoid blocking when using x10_gui */
static int handle_answer_from_server(int sock)
{
  static char buffer[MAXBUF];

  do
    {
      if(!read(sock,buffer,MAXBUF))
	return 1;
      printf(buffer);
    }
  while(strcmp(buffer,"OK\n"));

  fflush(stdout);
  return(0);
}


int main(int argc,char **argv)
{
  int sock;
  char buffer[MAXBUF];
  fd_set set_fds;
  struct timeval valtime;
  int select_result;

  if(argc != 3)
    usage();

  /* connection */
  if(!(sock=init_client(argv[1],atoi(argv[2]))))
    fatal_error("Error connect !\n");
  
  /* cycle on data both from stdin and our server */
    do
    {
      /* Wait for something both from stdin and our socket */
      valtime.tv_sec = CLIENT_TIME_OUT_SELECT_SECONDS;
      valtime.tv_usec = CLIENT_TIME_OUT_SELECT_USECONDS;
      FD_ZERO(&set_fds);
      FD_SET(fileno(stdin),&set_fds);
      FD_SET(sock,&set_fds);
      select_result = select(sock+1,&set_fds, NULL, NULL, &valtime);

      /* nothing, continue */
      if(select_result == -1)
	continue;

      /* something from the server, handle it. On error, the connection has
	 been broken */
      if(FD_ISSET(sock,&set_fds))
	if(handle_answer_from_server(sock))
	  fatal_error("Peer has died or connection problem !!\n");

      /* something from stdin, just pass it to the server */
      if(FD_ISSET(fileno(stdin),&set_fds))
	{
	  fgets(buffer,MAXBUF,stdin);
	  write(sock,buffer,strlen(buffer));
	}

    }
    while(!feof(stdin));

  close(sock);
  return(0);
}
